@@ -294,6 +294,7 @@ def shuffle(coll: Iterable[T]) -> Iterable[T]:
 
 
 def _iter(coll, n=0):
+    # If it's an iterator, we already consumed the beginning
     if isinstance(coll, collections.abc.Iterator):
         return coll
     return coll[n:]
@@ -443,24 +444,25 @@ def second(coll: Iterable[T]) -> Optional[T]:
     return first(rest(coll))
 
 
-def nth(coll, n, not_found=_nil):
+def nth(coll: Iterable[T], n: int, not_found: Any = _nil) -> Any:
     """
     Returns the value at the index. ``get`` returns ``None`` if the index is
     out of bounds, ``nth`` throws an exception unless ``not_found`` is
     supplied.  ``nth`` also works for strings, lists, tuples, and, in O(n)
     time, for other iterables.
     """
-    if hasattr(coll, "__getitem__"):
-        try:
-            return coll[n]
-        except IndexError:
-            if not_found is not _nil:
-                return not_found
-            raise
+    if n >= 0:
+        if hasattr(coll, "__getitem__"):
+            try:
+                return cast(list, coll)[n]
+            except IndexError:
+                if not_found is not _nil:
+                    return not_found
+                raise
 
-    for i, e in enumerate(coll):
-        if i == n:
-            return e
+        for i, e in enumerate(coll):
+            if i == n:
+                return e
 
     if not_found is _nil:
         raise IndexError("%s index out of range" % type(coll))
@@ -595,7 +597,7 @@ def repeatedly(f: Union[Callable[[], T2], int], n: Optional[Union[int, Callable[
         n -= 1
 
 
-def iterate(f: Callable, x) -> Iterable:
+def iterate(f: Callable[[Any], Any], x) -> Iterable:
     """
     Returns a generator of ``x``, ``f(x)``, ``f(f(x))``, etc.
     """
@@ -616,7 +618,7 @@ def repeat(x: T, n: Optional[int] = None) -> Iterable[T]:
     return itertools.repeat(x, **kwargs)
 
 
-def range(*args) -> Iterator[int]:
+def range(*args: int) -> Iterator[int]:
     """
     Usage: range()
            range(end)
@@ -628,8 +630,7 @@ def range(*args) -> Iterator[int]:
     ``1``, and ``end`` to infinity. When ``step`` is equal to ``0``, returns an
     infinite sequence of ``start``.
 
-    Note that this delegates to Python’s built-in ``range`` (or ``xrange`` in
-    Python 2) if there are arguments.
+    Note that this delegates to Python’s built-in ``range`` if there are arguments.
     """
     if args:
         for e in _range(*args):
